#version 430 core
struct Material {
    sampler2D texture_diffuse1;
    sampler2D texture_specular1;
    float shininess;
}; 



struct Light {
    vec3 direction;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

in vec3 fragPosition;
in vec3 Normal;
in vec2 TexCoords;
in vec4 fragPosLightSpace;




out vec4 color;
out vec3 tex;


uniform vec3 viewPosition;
uniform Material material;
uniform Light sun;
uniform sampler2D shadowMap;
uniform samplerCube skyBox;

uniform int hexColor;
uniform float gamma;




uniform float materialShininess;
uniform vec3 materialSpecularColor;

uniform float alphaValue = 1.0f;
const vec3 hexGreen = vec3(0.266,0.912,0.060);
const vec3 hexRed = vec3(0.850,0.086,0.004);
const vec3 hexBlack = vec3(0.912,0.808,0.198);
const float cel = 5.0;
float calculateShadows(vec4 fpls)
{
    // perform perspective divide
    vec3 projCoords = fpls.xyz / fpls.w;
    
    projCoords = projCoords * 0.5 + 0.5; //only range 0,1
    // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;

//	float bias = 0.007f;
	float bias = max(0.08 * (1.0 - dot(Normal, -sun.direction)), 0.005);
//    float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
		{
		for(int y = -1; y <= 1; ++y)
			{
			float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
			shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
			}    
		}
	

	shadow /= 20.0;
	if(projCoords.z > 1.0) { shadow = 0.0; }
    return shadow;
}

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}


void main()
{    
vec3 normal = normalize(Normal);
   vec3 lightDirection = normalize(-sun.direction);
	vec3 viewDirection = normalize(viewPosition - fragPosition);
	vec3 I = normalize(fragPosition - viewPosition);
	vec3 R = reflect(I, normal);
    vec3 reflectDirection = reflect(-lightDirection, normal);  
	//ambient

	tex =  vec3(mix(texture(material.texture_diffuse1, TexCoords),texture(skyBox,R),0.2));
	


	vec3 texHsv = rgb2hsv(tex);
	float level = floor(texHsv.z*cel);
	texHsv.z = level / cel;
	
	tex = hsv2rgb(texHsv); 



	
   vec3 ambient = sun.ambient * tex;

   
   
   // Diffuse
   float diff = max(dot(normal, lightDirection), 0.0);
   level = floor(diff * cel);
  diff = level / cel;
   vec3 diffuse = sun.diffuse * diff * tex;
     //rgb to hsv -> v -> brightness value cel shading
	//Spec

	


//	vec3 viewDirection = normalize(viewPosition - fragPosition);
 //   vec3 reflectDirection = reflect(-lightDirection, normal);  
    float spec = pow(max(dot(viewDirection, reflectDirection), 0.0), material.shininess);
	
	level = floor(spec * cel);
	spec = level / cel;

	vec3 surfaceToCamera;
	surfaceToCamera = normalize(viewDirection);

	


    vec3 specular = sun.specular * spec * tex;
	specular = max(min(specular,1.0),0.0);
    
	 float shadow = calculateShadows(fragPosLightSpace);
	color = vec4(ambient + (1.0 - shadow) * (diffuse + specular),alphaValue);
	
	
	//float gamma = 1.0f;

	// 0:default
	// 1:path/highlight
	// 2:green
	// 3:red

	if(hexColor==0) {
		color.rgb = pow(color.rgb, vec3(1.0/gamma));
	}
	else if(hexColor==1) {
		color.rgb = pow(color.rgb, vec3(1.0/(gamma+2.0)));
	}
	else if(hexColor==2) {
		color.rgb = mix(hexGreen,pow(color.rgb, vec3(1.0/gamma)),0.5);
		//color.rgb = pow(color.rgb, vec3(1.0/gamma));
	}
	else if(hexColor==3) {
		color.rgb = mix(hexRed,pow(color.rgb, vec3(1.0/gamma)),0.5);
	}
	else if(hexColor==6) {
		color.rgb = mix(hexBlack,pow(color.rgb, vec3(1.0/gamma)),0.1);
	}
	else {
		color.rgb = pow(color.rgb, vec3(1.0/gamma));
	}
	
	
	 
}




